use crate::server::ServerType;
use flate2::write::GzEncoder;
use flate2::Compression;
use std::fs::File;
use std::io;
use std::io::Write;
use std::path::{Path, PathBuf};
use std::process::Child;
use std::process::{Command, Stdio};

pub struct ServerProcess {
    type_: ServerType,
    version: String,
    config_dir: PathBuf,
    world_dir: PathBuf,
    backup_dir: PathBuf,
    child: Child,
}

impl ServerProcess {
    pub fn new(
        type_: ServerType,
        version: String,
        config_dir: PathBuf,
        world_dir: PathBuf,
        backup_dir: PathBuf,
        child: Child,
    ) -> ServerProcess {
        ServerProcess {
            type_,
            version,
            config_dir,
            world_dir,
            backup_dir,
            child,
        }
    }

    pub fn send_command(&mut self, cmd: &str) -> std::io::Result<()> {
        match cmd.trim() {
            "stop" | "exit" => self.stop()?,
            "backup" => self.backup()?,
            s => self.custom(s)?,
        }

        Ok(())
    }

    fn custom(&mut self, cmd: &str) -> std::io::Result<()> {
        let mut stdin = self.child.stdin.as_ref().unwrap();
        stdin.write_all(format!("{}\n", cmd.trim()).as_bytes())?;
        stdin.flush()?;

        Ok(())
    }

    pub fn stop(&mut self) -> std::io::Result<()> {
        self.custom("stop")?;
        self.child.wait()?;

        Ok(())
    }

    pub fn backup(&mut self) -> std::io::Result<()> {
        // Make sure the server isn't modifying the files during the backup
        self.custom("save-off")?;
        self.custom("save-all")?;

        // Create a gzip-compressed tarball of the worlds folder
        let filename = format!(
            "{}",
            chrono::offset::Local::now().format("%Y-%m-%d_%H-%M-%S.tar.gz")
        );
        let path = self.backup_dir.join(&filename);
        let tar_gz = std::fs::File::create(path)?;
        let enc = GzEncoder::new(tar_gz, Compression::default());
        let mut tar = tar::Builder::new(enc);

        tar.append_dir_all("worlds", &self.world_dir)?;

        // We don't store all files in the config, as this would include caches
        tar.append_path_with_name(self.config_dir.join("server.properties"), "config/server.properties")?;


        // We add a file to the backup describing for what version it was made
        let info = format!("{} {}", self.type_, self.version);
        let info_bytes = info.as_bytes();
        let mut header = tar::Header::new_gnu();
        header.set_size(info_bytes.len().try_into().unwrap());

        tar.append_data(&mut header, "info.txt", info_bytes)?;

        // tar.append_dir_all("config", &self.config_dir)?;
        //
        // Backup file gets finalized in the drop

        self.custom("save-on")
    }
}
